/**************************************************************************
***    
*** Copyright (c) 2004 Regents of the University of Michigan,
***               Hayward H. Chan and Igor L. Markov
***
***  Contact author(s): hhchan@umich.edu, imarkov@umich.edu
***  Original Affiliation:   EECS Department, 
***                          The University of Michigan,
***                          Ann Arbor, MI 48109-2122
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/
#ifndef DEBUG_H
#define DEBUG_H

#include "datastrbacksoftst.h"
#include "datastrfrontsoftst.h"

#include <iostream>

// --------------------------------------------------------
class Debug
{
public:   
   static void GnuplotHeading(ostream& outs,
                              const string& filename);
   static void OutputBoundary(int format,
                              ostream& outs,
                              const BoundaryType& boundary);
   
   static void ReadCurve(istream& ins,
                         vector<Point>& curve);
   static void PlotCurves(ostream& outs,
                          const vector< vector<Point> >& curves);
   static void initialize(const string& nName);

   static void OutputNode(ostream& outs, const SoftNode& node);
   static void OutputSoftSTree(ostream& outs, const SoftSTree& st);
   static void OutputSoftSliceRecord(ostream& outs, const SoftSliceRecord& ssr);
   static void CheckSTree(const SoftSTree& sst);
   
   enum FormatType {GNUPLOT = 0, CONSOLE};
   static const string TEMP_FILE_NAME;

private:
   Debug();
   static int count;
   static string filePrefix;
   static string fileSuffix;
};
// --------------------------------------------------------
inline ostream& operator <<(ostream& outs, const SoftNode& node)
{
   Debug::OutputNode(outs, node);
   return outs;
}
// --------------------------------------------------------
#endif
